==> sqlitedbms-0.1.4.tar.gz: sqlitedbms-0.1.4/src/backend/libnetwork.c <==
#include "include/common.h"

int server_sock_init( sock_t *server_addr, short port, int backlog )
{
	int	sockfd;
	int	on;

	server_addr->sa.sa_family = AF_INET;
	server_addr->sa_in.sin_addr.s_addr = htonl( INADDR_ANY );
	server_addr->sa_in.sin_port = htons( port );

	sockfd = socket( AF_INET, SOCK_STREAM, 0 );
        if ( sockfd < 0 )
                return -1;

	(void) fcntl( sockfd, F_SETFD, 1 );
        (void) fcntl( sockfd, F_SETFL, O_NONBLOCK|O_NDELAY );

        on = 1;
        (void) setsockopt( sockfd, SOL_SOCKET, SO_REUSEADDR, (char*) &on, sizeof(on) );
        if ( bind( sockfd, &server_addr->sa, sizeof(struct sockaddr_in) ) < 0 )
        {
                close( sockfd );
                return -1;
        }

	if ( listen( sockfd, backlog ) < 0 )
        {
                close( sockfd );
                return -1;
        }

	return sockfd;
}

int net_send( int sock, char *buf, int len )
{
	int		offset = 0;
	int		res;
	struct pollfd	pollfd;

	pollfd.fd = sock;
	pollfd.events = POLLOUT;
	while ( offset < len )
	{
		res = poll( &pollfd, 1, 1000 );
		if ( res > 0 )
		{
			res = write( sock, &(buf[offset]), len-offset );
			if ( res < 0 )
			{
				return SQLITE_ERROR_SEND;
			}
			offset += res;
		}
		else if ( res < -1 )
			return SQLITE_ERROR_SEND;
	}
	return SQLITE_OK;
}

int net_sendfile( int sock, int filefd, off_t offset, ssize_t size )
{
	long		sockopt;
	ssize_t		res;
	ssize_t		len;
	ssize_t		sended = 0;

	struct pollfd	pollfd;
	int		pollres;

	pollfd.fd = sock;
	pollfd.events = POLLOUT;

	fcntl( sock, F_SETFL, sockopt ^ O_NONBLOCK );

	while ( size > sended )
	{
		pollres = poll( &pollfd, 1, 1000 );
		if ( pollres == 0 )
			continue;
		else if ( pollres < -1 )
			return SQLITE_ERROR;

		len = size - sended;
		res = sendfile( sock, filefd, &offset, len );
		if ( res <= 0 )
		{
			fcntl( sock, F_SETFL, sockopt | O_NONBLOCK );
			return SQLITE_ERROR;
		}
		sended += res;
	}
	fcntl( sock, F_SETFL, sockopt | O_NONBLOCK );

	return SQLITE_OK;
}

==> sqlitedbms-0.1.6.tar.gz: sqlitedbms-0.1.6/src/backend/libnetwork.c <==
#include "include/common.h"

int server_sock_init( sock_t *server_addr, short port, int backlog )
{
	int	sockfd;
	int	on;

	server_addr->sa.sa_family = AF_INET;
	server_addr->sa_in.sin_addr.s_addr = htonl( INADDR_ANY );
	server_addr->sa_in.sin_port = htons( port );

	sockfd = socket( AF_INET, SOCK_STREAM, 0 );
        if ( sockfd < 0 )
                return -1;

	(void) fcntl( sockfd, F_SETFD, 1 );
        (void) fcntl( sockfd, F_SETFL, O_NONBLOCK|O_NDELAY );

        on = 1;
        (void) setsockopt( sockfd, SOL_SOCKET, SO_REUSEADDR, (char*) &on, sizeof(on) );
        if ( bind( sockfd, &server_addr->sa, sizeof(struct sockaddr_in) ) < 0 )
        {
                close( sockfd );
                return -1;
        }

	if ( listen( sockfd, backlog ) < 0 )
        {
                close( sockfd );
                return -1;
        }

	return sockfd;
}

int net_send( int sock, char *buf, int len )
{
	int		offset = 0;
	int		res;
	struct pollfd	pollfd;

	pollfd.fd = sock;
	pollfd.events = POLLOUT;
	while ( offset < len )
	{
		res = poll( &pollfd, 1, 1000 );
		if ( res > 0 )
		{
			res = write( sock, &(buf[offset]), len-offset );
			if ( res < 0 )
			{
				return SQLITE_ERROR_SEND;
			}
			offset += res;
		}
		else if ( res < -1 )
			return SQLITE_ERROR_SEND;
	}
	return SQLITE_OK;
}

int net_sendfile( int sock, int filefd, off_t offset, ssize_t size )
{
	long		sockopt;
	ssize_t		res;
	ssize_t		len;
	ssize_t		sended = 0;

	struct pollfd	pollfd;
	int		pollres;

	pollfd.fd = sock;
	pollfd.events = POLLOUT;

	//fcntl( sock, F_SETFL, sockopt ^ O_NONBLOCK );

	while ( size > sended )
	{
		pollres = poll( &pollfd, 1, 1000 );
		if ( pollres == 0 )
			continue;
		else if ( pollres < -1 )
			return SQLITE_ERROR;

		len = size - sended;
		res = sendfile( sock, filefd, &offset, len );
		if ( res <= 0 )
		{
			//fcntl( sock, F_SETFL, sockopt | O_NONBLOCK );
			return SQLITE_ERROR;
		}
		sended += res;
	}
	//fcntl( sock, F_SETFL, sockopt | O_NONBLOCK );

	return SQLITE_OK;
}
